import type { TransformerLLM } from "./transformer-llm";
export type MetadataBaseValue = string | number | bigint | boolean;
export type MetadataValue = MetadataBaseValue | MetadataBaseValue[] | MetadataValue[];
export type Version = 1 | 2 | 3;
export declare enum GGMLQuantizationType {
    F32 = 0,
    F16 = 1,
    Q4_0 = 2,
    Q4_1 = 3,
    Q5_0 = 6,
    Q5_1 = 7,
    Q8_0 = 8,
    Q8_1 = 9,
    Q2_K = 10,
    Q3_K = 11,
    Q4_K = 12,
    Q5_K = 13,
    Q6_K = 14,
    Q8_K = 15,
    IQ2_XXS = 16,
    IQ2_XS = 17,
    IQ3_XXS = 18,
    IQ1_S = 19,
    IQ4_NL = 20,
    IQ3_S = 21,
    IQ2_S = 22,
    IQ4_XS = 23,
    I8 = 24,
    I16 = 25,
    I32 = 26,
    I64 = 27,
    F64 = 28,
    IQ1_M = 29,
    BF16 = 30
}
export declare enum GGUFValueType {
    UINT8 = 0,
    INT8 = 1,
    UINT16 = 2,
    INT16 = 3,
    UINT32 = 4,
    INT32 = 5,
    FLOAT32 = 6,
    BOOL = 7,
    STRING = 8,
    ARRAY = 9,
    UINT64 = 10,
    INT64 = 11,
    FLOAT64 = 12
}
declare const ARCHITECTURES: readonly ["llama", "falcon", "grok", "gpt2", "gptj", "gptneox", "mpt", "baichuan", "starcoder", "persimmon", "refact", "bert", "nomic-bert", "bloom", "stablelm", "qwen", "qwen2", "qwen2moe", "phi2", "phi3", "plamo", "codeshell", "orion", "internlm2", "minicpm", "gemma", "starcoder2", "mamba", "xverse", "command-r", "dbrx", "olmo", "rwkv", "whisper"];
export type Architecture = (typeof ARCHITECTURES)[number];
export interface GGUFGeneralInfo<TArchitecture extends Architecture> {
    "general.architecture": TArchitecture;
    "general.name"?: string;
    "general.file_type"?: number;
    "general.quantization_version"?: number;
}
type ModelMetadata = Whisper | RWKV | TransformerLLM;
interface NoModelMetadata {
    "general.architecture"?: undefined;
}
export type ModelBase<TArchitecture extends Architecture | `encoder.${Extract<Architecture, "whisper">}` | `decoder.${Extract<Architecture, "whisper">}`> = Record<`${TArchitecture}.context_length` | `${TArchitecture}.block_count` | `${TArchitecture}.embedding_length` | `${TArchitecture}.feed_forward_length`, number>;
type TokenizerModel = "no_vocab" | "llama" | "gpt2" | "bert";
interface Tokenizer {
    "tokenizer.ggml.model": TokenizerModel;
    "tokenizer.ggml.tokens": string[];
    "tokenizer.ggml.scores": number[];
    "tokenizer.ggml.token_type": number[];
    "tokenizer.ggml.bos_token_id": number;
    "tokenizer.ggml.eos_token_id": number;
    "tokenizer.ggml.add_bos_token": boolean;
    "tokenizer.chat_template"?: string;
}
interface NoTokenizer {
    "tokenizer.ggml.model"?: undefined;
}
export type RWKV = GGUFGeneralInfo<"rwkv"> & ModelBase<"rwkv"> & {
    "rwkv.architecture_version": number;
};
export type Whisper = GGUFGeneralInfo<"whisper"> & ModelBase<"encoder.whisper"> & ModelBase<"decoder.whisper"> & {
    "whisper.encoder.mels_count": number;
    "whisper.encoder.attention.head_count": number;
    "whisper.decoder.attention.head_count": number;
};
export interface GGUFMetadataOptions {
    /**
     * Enable strict type for known GGUF fields.
     *
     * @default true
     */
    strict: boolean;
}
export type GGUFMetadata<Options extends GGUFMetadataOptions = {
    strict: true;
}> = {
    version: Version;
    tensor_count: bigint;
    kv_count: bigint;
} & GGUFModelKV & (Options extends {
    strict: true;
} ? unknown : Record<string, MetadataValue>);
export type GGUFModelKV = (NoModelMetadata | ModelMetadata) & (NoTokenizer | Tokenizer);
export interface GGUFTensorInfo {
    name: string;
    n_dims: number;
    shape: bigint[];
    dtype: GGMLQuantizationType;
    offset: bigint;
}
export interface GGUFParseOutput<Options extends GGUFMetadataOptions = {
    strict: true;
}> {
    metadata: GGUFMetadata<Options>;
    tensorInfos: GGUFTensorInfo[];
}
export {};
//# sourceMappingURL=types.d.ts.map